if(NOT GMP_CONFIG_INCLUDED)
set(GMP_CONFIG_INCLUDED 1)

set(GMP_VERSION 6.2.0)

include(Common)

if(CMAKE_VERSION VERSION_GREATER_EQUAL "3.12")
	# find_package() uses <PackageName>_ROOT implicit hints
	# https://cmake.org/cmake/help/latest/policy/CMP0074.html
	cmake_policy(PUSH)
	cmake_policy(SET CMP0074 NEW)
endif()	

external_project_directories(gmp ${GMP_VERSION} GMP_TMP_DIR GMP_STAMP_DIR GMP_DOWNLOAD_DIR GMP_SOURCE_DIR GMP_BINARY_DIR GMP_INSTALL_DIR)
append_target_architecture_directory(${GMP_TMP_DIR} GMP_TMP_DIR)
append_target_architecture_directory(${GMP_STAMP_DIR} GMP_STAMP_DIR)
append_target_architecture_directory(${GMP_BINARY_DIR} GMP_BINARY_DIR)
append_target_architecture_directory(${GMP_INSTALL_DIR} GMP_INSTALL_DIR)

if(NOT GMP_ROOT)
	set(GMP_ROOT ${GMP_INSTALL_DIR})
endif()

function(gmp_configure_command RESULT)
    msys_path(${GMP_SOURCE_DIR} GMP_MSYS_SOURCE_DIR)
    file(RELATIVE_PATH RELATIVE_GMP_SOURCE_DIR ${GMP_BINARY_DIR} ${GMP_SOURCE_DIR}) 
    set(CFLAGS "${CXX_ARCH}")
    #set(libgmp_la_LDFLAGS "-no-undefined")
    #set(GMP_LDFLAGS "-no-undefined")
    # configure must be called in relative path
    set(CONFIGURE_COMMAND ${RELATIVE_GMP_SOURCE_DIR}/configure --prefix=${GMP_INSTALL_DIR} --srcdir=${RELATIVE_GMP_SOURCE_DIR} --build=${TARGET_ARCHITECTURE} --enable-static --disable-shared)
    set(${RESULT} ${CONFIGURE_COMMAND} CFLAGS=${CFLAGS} PARENT_SCOPE)
endfunction()

function(install_gmp)
	message(STATUS "Install GMP Library ${GMP_VERSION}")
	
	set(GMP_URL "ftp://ftp.gnu.org/gnu/gmp/gmp-${GMP_VERSION}.tar.bz2")
	if(GMP_VERSION VERSION_EQUAL 6.0.0a)
	    set(GMP_MD5 b7ff2d88cae7f8085bd5006096eed470)
	endif() 
	
	gmp_configure_command(GMP_CONFIGURE_COMMAND)
	message(STATUS "GMP_CONFIGURE_COMMAND: ${GMP_CONFIGURE_COMMAND}")

	set(GMP_EXTERNAL gmp_external)
 	set(DEPENDENCIES ${DEPENDENCIES} ${GMP_EXTERNAL} PARENT_SCOPE)
	
	ExternalProject_Add(${GMP_EXTERNAL}
		TMP_DIR ${GMP_TMP_DIR}
		STAMP_DIR ${GMP_STAMP_DIR}
		DOWNLOAD_DIR ${GMP_DOWNLOAD_DIR}
		SOURCE_DIR ${GMP_SOURCE_DIR}
		BINARY_DIR ${GMP_BINARY_DIR}
		INSTALL_DIR ${GMP_INSTALL_DIR}
		
		URL ${GMP_URL}
		URL_MD5 ${GMP_MD5}
		CONFIGURE_COMMAND ${GMP_CONFIGURE_COMMAND}
		BUILD_COMMAND $(MAKE)
		INSTALL_COMMAND $(MAKE) install
		TEST_COMMAND $(MAKE) check)
endfunction()

macro(require_gmp)
    find_package(GMP REQUIRED)
 	include_directories(${GMP_INCLUDE_DIR})
endmacro()

macro(find_gmp)
    find_package(GMP)
    if(NOT GMP_FOUND)
    	install_gmp()
    	set(GMP_INCLUDE_DIR ${GMP_INSTALL_DIR}/include)
    	set(GMP_LIBRARY ${GMP_INSTALL_DIR}/lib/libgmp.a)
    endif()
	include_directories(${GMP_INCLUDE_DIR})
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_gmp
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GMP_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GMP_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GMP_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GMP_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GMP_STAMP_DIR})	

if(CMAKE_VERSION VERSION_GREATER_EQUAL "3.12")
	cmake_policy(POP)
endif()	

endif()
